package de.uni.karlsruhe.aifb.prog2.chess;

/** Basis-Klasse der meisten Schachfiguren. Der LinearChessman ist
  * in der Lage, sich in gerader Linie (linear) fortzubewegen. Dies
  * trifft, ausser beim Pferd, auf alle Figuren zu.
  **/
public class LinearChessman extends AbstractChessman {

  /** Besagt, dass sich die Figur geradlinig bewegen laesst. */
  private boolean horizontalAndVertical;

  /** Besagt, dass sich die Figur diagonal bewegen laesst. */
  private boolean diagonal;

  /** Reichweite einer Figur in Feldern. */
  private int range;

  /** Gerade Richtungen */
  private final static int[][] DIRECTIONS_HV =
    {{1,0},{-1,0},{0,1},{0,-1}};

  /** Diagonale Richtungen */
  private final static int[][] DIRECTIONS_DIAG =
    {{1,1},{1,-1},{-1,1},{-1,-1}};

  /** Konstruktor.
    * @param isBlack ist true genau dann, wenn die Figur schwarz
    *  ist.
    * @param description eine Darstellung der Figur als Zeichen.
    *  Dieser Wert muss ein Grossbuchstabe (zwischen A und Z) sein.
    * @param horizontalAndVertical ist true genau dann, wenn sich
    *   die Figur horizontal und vertikal bewegen kann
    * @param diagonal ist true genau dann, wenn sich die Figur
    *   diagonal bewegen kann.
    * @param range die Anzahl der Felder, die die Figur maximal
    *   pro Zug ziehen kann
    **/
  public LinearChessman(boolean isBlack,char description,
       boolean horizontalAndVertical, boolean diagonal,
       int range) {
    super(isBlack,description);
    this.horizontalAndVertical = horizontalAndVertical;
    this.diagonal = diagonal;
    this.range = range;
  }

  /** Ueberprueft eine Zugrichtung */
  private void check(int[] direction,Chessboard board,
                     int row,int col,boolean[][] destinations) {
    try
    {
      for (int i = 1; i <= range; i++) {
        // Gehe einen Schritt
        row = row + direction[0];
        col = col + direction[1];
        // Markiere das Feld als Kandidaten
        destinations[row][col] = true;
        // Ist das Feld schon besetzt?
        if (board.getContent(row,col) != null)
          break;
      }
    }
    catch(ArrayIndexOutOfBoundsException e)
    {
      // Wir sind am Rande des Feldes angekommen!
    }
  }


  /** Ueberprueft, auf welche Felder sich die Figur im naechsten Zug
    * theoretisch bewegen kann.
    **/
  public boolean[][] getDestinations
    (Chessboard board,int row,int col) {
    // Bis das Gegenteil bewiesen ist,
    // darf unsere Figur nirgends hin
    boolean[][] res =
      new boolean[board.getNumOfRows()]
                 [board.getNumOfColumns()];
    // In einer Schleife berechnen wir alle Positionen,
    // die theoretisch moeglich sind.
    if (horizontalAndVertical)
      for (int i = 0; i < DIRECTIONS_HV.length; i++)
        check(DIRECTIONS_HV[i],board,row,col,res);
    if (diagonal)
      for (int i = 0; i < DIRECTIONS_DIAG.length; i++)
        check(DIRECTIONS_DIAG[i],board,row,col,res);
    // Eliminiere nun die Felder,
    // die nicht betreten werden koennen
    cleanBlocked(res,board,this);
    // Gib das Ergebnis zurueck
    return res;
  }
}